@@ -61,6 +61,8 @@
 #include "gsturi.h"
 #include "gstregistry.h"
 
+#include "glib-compat-private.h"
+
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
@@ -169,13 +171,10 @@ void
 __gst_element_details_clear (GstElementDetails * dp)
 {
   g_free (dp->longname);
-  dp->longname = NULL;
   g_free (dp->klass);
-  dp->klass = NULL;
   g_free (dp->description);
-  dp->description = NULL;
   g_free (dp->author);
-  dp->author = NULL;
+  memset (dp, 0, sizeof (GstElementDetails));
 }
 
 #define VALIDATE_SET(__dest, __src, __entry)                            \
@@ -220,10 +219,23 @@ gst_element_factory_cleanup (GstElementFactory * factory)
 
   for (item = factory->staticpadtemplates; item; item = item->next) {
     GstStaticPadTemplate *templ = item->data;
+    GstCaps *caps = (GstCaps *) & (templ->static_caps);
 
-    g_free (templ->name_template);
     g_free ((gchar *) templ->static_caps.string);
-    memset (&(templ->static_caps), 0, sizeof (GstStaticCaps));
+
+    /* FIXME: this is not threadsafe */
+    if (caps->refcount == 1) {
+      GstStructure *structure;
+      guint i;
+
+      for (i = 0; i < caps->structs->len; i++) {
+        structure = (GstStructure *) gst_caps_get_structure (caps, i);
+        gst_structure_set_parent_refcount (structure, NULL);
+        gst_structure_free (structure);
+      }
+      g_ptr_array_free (caps->structs, TRUE);
+      caps->refcount = 0;
+    }
     g_free (templ);
   }
   g_list_free (factory->staticpadtemplates);
@@ -242,7 +254,8 @@ gst_element_factory_cleanup (GstElementFactory * factory)
 
 /**
  * gst_element_register:
- * @plugin: #GstPlugin to register the element with
+ * @plugin: #GstPlugin to register the element with, or NULL for a static
+ * element (note that passing NULL only works in GStreamer 0.10.13 and later)
  * @name: name of elements of this type
  * @rank: rank of element (higher rank means more importance when autoplugging)
  * @type: GType of element to register
@@ -282,7 +295,7 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstStaticPadTemplate *newt;
 
     newt = g_new0 (GstStaticPadTemplate, 1);
-    newt->name_template = g_strdup (templ->name_template);
+    newt->name_template = g_intern_string (templ->name_template);
     newt->direction = templ->direction;
     newt->presence = templ->presence;
     newt->static_caps.string = gst_caps_to_string (templ->caps);
@@ -297,12 +310,19 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstURIHandlerInterface *iface = (GstURIHandlerInterface *)
         g_type_interface_peek (klass, GST_TYPE_URI_HANDLER);
 
-    if (!iface || !iface->get_type || !iface->get_protocols)
+    if (!iface || (!iface->get_type && !iface->get_type_full) ||
+        (!iface->get_protocols && !iface->get_protocols_full))
       goto urierror;
-    factory->uri_type = iface->get_type ();
+    if (iface->get_type)
+      factory->uri_type = iface->get_type ();
+    else if (iface->get_type_full)
+      factory->uri_type = iface->get_type_full (factory->type);
     if (!GST_URI_TYPE_IS_VALID (factory->uri_type))
       goto urierror;
-    factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    if (iface->get_protocols)
+      factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    else if (iface->get_protocols_full)
+      factory->uri_protocols = iface->get_protocols_full (factory->type);
     if (!factory->uri_protocols)
       goto urierror;
   }
@@ -313,7 +333,11 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
   g_free (interfaces);
 
-  GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
+  if (plugin && plugin->desc.name) {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = plugin->desc.name;
+  } else {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = "NULL";
+  }
   gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
   GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
 
@@ -379,14 +403,18 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
 
   /* create an instance of the element, cast so we don't assert on NULL */
   element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
-  if (element == NULL)
+  if (G_UNLIKELY (element == NULL))
     goto no_element;
 
   /* fill in the pointer to the factory in the element class. The
-   * class will not be unreffed currently. */
+   * class will not be unreffed currently. 
+   * FIXME: This isn't safe and may leak a refcount on the factory if 2 threads
+   * create the first instance of an element at the same moment */
   oclass = GST_ELEMENT_GET_CLASS (element);
-  if (oclass->elementfactory == NULL)
+  if (G_UNLIKELY (oclass->elementfactory == NULL))
     oclass->elementfactory = factory;
+  else
+    gst_object_unref (factory);
 
   if (name)
     gst_object_set_name (GST_OBJECT (element), name);
@@ -404,11 +432,13 @@ load_failed:
 no_type:
   {
     GST_WARNING_OBJECT (factory, "factory has no type");
+    gst_object_unref (factory);
     return NULL;
   }
 no_element:
   {
     GST_WARNING_OBJECT (factory, "could not create element");
+    gst_object_unref (factory);
     return NULL;
   }
 }
@@ -596,7 +626,7 @@ __gst_element_factory_add_interface (GstElementFactory * elementfactory,
  * gst_element_factory_get_static_pad_templates:
  * @factory: a #GstElementFactory
  *
- * Gets the #GList of padtemplates for this factory.
+ * Gets the #GList of #GstStaticPadTemplate for this factory.
  *
  * Returns: the padtemplates
  */
@@ -642,3 +672,31 @@ gst_element_factory_get_uri_protocols (GstElementFactory * factory)
 
   return factory->uri_protocols;
 }
+
+/**
+ * gst_element_factory_has_interface:
+ * @factory: a #GstElementFactory
+ * @interfacename: an interface name
+ *
+ * Check if @factory implements the interface with name @interfacename.
+ *
+ * Returns: #TRUE when @factory implement the interface.
+ *
+ * Since: 0.10.14
+ */
+gboolean
+gst_element_factory_has_interface (GstElementFactory * factory,
+    const gchar * interfacename)
+{
+  GList *walk;
+
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), FALSE);
+
+  for (walk = factory->interfaces; walk; walk = g_list_next (walk)) {
+    gchar *iname = (gchar *) walk->data;
+
+    if (!strcmp (iname, interfacename))
+      return TRUE;
+  }
+  return FALSE;
+}
